#!/bin/bash
#
##################
##### LiHAS ######
##################
#
# rbe, Aug. 2010, parts from aptchku.sh
#
# Beschreibung: liefert Paketliste mit dpkg -l
#               ueberprueft Hostsystem und virtuelle Server mit
#               anschliessendem Versenden einer E-Mail 

set -u
#### VARIABLEN

export LANG=C

HOSTNAME="$(hostname)"

# Programme, die nicht im Paket "coreutils" enthalten sind
AWK=awk
MUTT=mutt
SED=sed


# auf Vserver bezogene Variablen
VS_CONFIG_DIR=/etc/vservers
#VS_DIR=/var/lib/vservers
VS_DIR=`readlink $VS_CONFIG_DIR/.defaults/vdirbase`
VS_EXE=vserver

# temporaere Datei (beinhaltet Namen der Upgrade-Pakete)
TMPFILE=$(mktemp /tmp/`basename $0 .sh`-tmp.XXXXXX)
TMPERRFILE=$(mktemp /tmp/`basename $0 .sh`-err.XXXXXX)

CONFIGFILE="/etc/aptchk4u/aptchk4u.rc"
MUTTCONFIG="/etc/aptchk4u/muttrc"

### FUNKTIONEN:

# vserver Hauptfunktion: diese Funktion fragt den Status der aktiven,
# virtuellen Server ab und aktualisiert dann mittels der 1.
# Unterfunktion die Paketlisten der einzelnen virtuellen Server;
# mittels der 2. Unterfunktion wird eine Liste der benoetigten Pakete
# erstellt und anschliessend eine E-Mail je virtuellem Server verschickt

function check_vs_run() {
  echo -e "running vservers: "`ls /var/run/vservers`"\n" >>$TMPFILE
  cd $VS_DIR
     ls $VS_CONFIG_DIR/ | while read a; do
        vserver "$a" running 2> /dev/null 
        if [ "$?" == 0 ]; then
          rm -f "$TMPERRFILE"
          aptversion $a
          packetlist $a
        fi
    done
 }


function aptversion() {
  vs_host="$1"
  vserver="$VS_EXE $vs_host exec" 
  DISTRI=unkown
  debianv=0
  APT_G=false
  version=""
  if $vserver [ -e /etc/debian_version ]; then
    version=`$vserver cat /etc/debian_version`
    DISTRI=debian
  # Check if sarge or etch, for sarge use apt-get
    debianv=$($vserver cat /etc/debian_version | cut -d'.' -f1| cut -d'/' -f1)
    case "$debianv" in
    sarge) debianv=3;;
    etch)  debianv=4;;
    lenny) debianv=5;;
    [0-9]) true ;;
    *) debianv=6;;
    esac

    if [ "$debianv" -gt 3  ]
    then
       APT_G="aptitude -o Aptitude::CmdLine::Ignore-Trust-Violations=true"
    else
       APT_G=apt-get
    fi
    apt_p=$($vserver which ${APT_G%% *})
    $vserver [ -x $apt_p ] || APT_G=false
  elif  $vserver [ -e /etc/redhat-release ]; then
     DISTRI=redhat
     redhatv=`$vserver cat /etc/redhat-release`
     version=`$vserver cat /etc/redhat-release`
  else
     DISTRI=unkown
     debianv=0
     APT_G=false
  fi
}
  



# vserver 2. Unterfunktion: erstellt Liste aller installierten  Pakete nd haengt
# diese an eine temporaere Datei an

function packetlist () {
  local packages=''
  local err=''
  local vs_hostname=$1
  local vs_rootfs_device='' vs_rootfs_type='' vs_rootfs_uuid=''
  #if $VS_EXE $a exec [ -e /etc/hostname ]; then
  #  VS_HOSTNAME=$($VS_EXE $a exec cat /etc/hostname)
  #else
  #  VS_HOSTNAME=$a
  #fi
  case "$DISTRI" in 
  debian)
    echo  "vname: $vs_hostname" >> $TMPFILE
    echo  "version: $version" >> $TMPFILE
    read vs_rootfs_device vs_rootfs_type rest <<<`df -TP /etc/vservers/$1/vdir | grep -v Filesystem | grep -v grep | tail -1`
    case "$vs_rootfs_type" in
    ext?) 
        vs_rootfs_uuid=`tune2fs -l $vs_rootfs_device | grep '^Filesystem UUID:' | grep -v grep | awk '{print $NF}'`
        echo "uuid: $vs_rootfs_uuid" >>$TMPFILE
    ;;
    esac
    $VS_EXE $a exec dpkg -l  2>&1 | awk '{printf("%-5s%s (%s)\n",$1,$2,$3)}' 2>&1 | tail -n +5  >>$TMPFILE 2>&1 
    if [ "$debianv" == 0 ]; then
      echo -e "ERROR:    missing /etc/debian_version\n" >> $TMPFILE
    fi
  ;;
  redhat)
    #packages=$($VS_EXE $a exec  yum check-update | grep ' base \| updates ' | awk '{print $1}')
    echo -e "vserver $vs_hostname($version):   "$packages"\n" | fmt >> $TMPFILE
    #echo 'todo' >> $TMPFILE
    $VS_EXE $a rpm -qa  >> $TMPFILE 2>&1
    
  ;;
  *)
    echo -e "vserver $vs_hostname: unkown distri\n" >>$TMPFILE 
  esac
  echo '-----' >> $TMPFILE
}

# Host-Funktion: aktualisiert die Paketlisten des Hosts, erstellt eine Liste
# der benoetigten Pakete und haengt diese an eine temporaere Datei an

function check_host_pack() {
  local rootfs_device='' rootfs_type='' rootfs_uuid=''
  
  #vserver=
  #aptversion ""
  local version=`cat /etc/debian_version`
    #echo -e "$HOSTNAME($version):" > $TMPFILE
    rm -f $TMPFILE
    echo "name: $HOSTNAME" >> $TMPFILE
    echo  "version: $version" >> $TMPFILE
    read rootfs_device rootfs_type rest <<<`df -TP /  | grep -v Filesystem | grep -v grep | tail -1`
    case "$rootfs_type" in
    ext?)
        rootfs_uuid=`tune2fs -l $rootfs_device | grep '^Filesystem UUID:' | grep -v grep | awk '{print $NF}'`
        echo "uuid: $rootfs_uuid" >>$TMPFILE
    ;;
    esac

    dpkg -l   2>&1 | awk '{printf("%-5s%s (%s)\n",$1,$2,$3)}' 2>&1 | tail -n +5 >>$TMPFILE 2>&1
    echo '-----'  >>$TMPFILE
}

# Mailfunktion: verschickt den Inhalt der temporaeren Datei, wenn Inhalt > 0

function mail_tmpfile() {
  if [ -s "$TMPFILE" ]; then
    gzip $TMPFILE && echo 's. Anhang' | $MUTT -F $MUTTCONFIG -s "$name_of_company: Packetlist on host \"$HOSTNAME\"" -a $TMPFILE.gz -- $email_address
  else
    echo "temporary file not found or zero content or no gzip" > /dev/null
  fi
}

###


### MAIN:

. $CONFIGFILE

# 1. Parameter ist E-Mail-Adresse, wenn vorhanden;
# ansonsten E-Mail-Adresse aus Konfigurationsdatei
if [ $# -gt 0 ] ; then
  email_address="$1"
fi

check_host_pack

# ueberpruefe Existenz des vserver Verzeichnissses unter /
# und fuehre erst dann die vserver Hauptfunktion aus

if [ -d "$VS_DIR" ]; then
  check_vs_run
else
  echo "vservers directory not found" > /dev/null
fi

mail_tmpfile

rm -f $TMPFILE{,.gz}
rm -f $TMPERRFILE
