#! /bin/bash

# export UCF_FORCE_CONFFOLD=1 
# vserver-dist-upgrade.sh [ '-y -o Dpkg::Options::=--force-confdef -o Dpkg::Options::=--force-confold' ]


CHECK_SERVICES=${CHECK_SERVICES:-true}
RESTART_SERVICES=${RESTART_SERVICES:-false}
CHECK_ETC=${CHECK_ETC:-true}

VSERVER_CONFIG_DIR="/etc/vservers"

export ABORT_CODE=99
export LANG=C
export LC_ALL=C

set -u

update_fehler_liste=""
upgrade_fehler_liste=""
started_services=""
interrupted=false
apt_args=()
vservers=""
#[ $# -gt 0 ] && args=$1

while [ $# -gt 0 ]; do
	case "$1" in
		-*) apt_args+=("$1")
		;;
		*'['*|*\**) vservers+=' '`ls -d $VSERVER_CONFIG_DIR/$1 | sed "s#$VSERVER_CONFIG_DIR/##"`
		;;
		[a-z]*) vservers+="$1 "
		;;
		*)	echo "Argumentfehler"
			exit1	
		;;
	esac
	shift
done


if $CHECK_SERVICES ; then
	services_before=`mktemp -t vserver-dist-upgrade.services_before.XXXX`
	services_after=`mktemp -t vserver-dist-upgrade.services_after.XXXX`
	services_diff=`mktemp -t vserver-dist-upgrade.services_diff.XXXX`
fi

endausgabe_func () {
        echo
        [ -n "$update_fehler_liste" ] && echo "Update-Fehler:$update_fehler_liste"
        [ -n "$upgrade_fehler_liste" ] && echo "Upgrade-Fehler:$upgrade_fehler_liste"
        [ -n "$started_services" ] && echo "Dienste neu gestartet: $started_services"
	if $CHECK_SERVICES ; then
		rm -f $services_before
		rm -f $services_after
		rm -f $services_diff
	fi
}

ask_cont () {
	echo
	echo -n "$v: "
	echo $*
        while read -p 'weiter?(Y|n) ' ; do
		REPLY=${REPLY:0:1}
                case "$REPLY" in
                        y|Y|j|J|'') 
                                return 0
                        ;;
                        n|N)
                                endausgabe_func
				exit
                        ;;
                esac
        done
}

int_func () {
	interrupted=true
	ask_cont "Unterbrochen durch Ctrl-C"
}

#vprefix_func () {
#	if [ -z "$v" ] ; then
#		echo 
#	else
#		echo "vserver $v exec"
#	fi
#}

search_running_services_func () {
	#vps  cft '?' | grep -v '\\_' | awk '{print $2,$NF}' | grep ^$context | awk '{print $NF}' | sort -u
	#netstat -xlnp  | `vprefix_func` bash | tail -n +3 | awk '{print $(NF-1)}' | cut -d'/' -f2 | sort -u
	EXCLUDE_SERVICES='logger screen bash smtp smtpd debian-start'
	local listen_services=`echo 'netstat -tulnp' | $vprefix bash | tail -n +3 | awk '{print $7}' | cut -d'/' -f1 | grep -v -- '-' | grep -v '^$'`
	if [ -n "$listen_services" ] ; then
		listen_services=`$vprefix ps --no-heading -o comm $listen_services`
	else
		listen_services=""
	fi
	sort -u <(echo 'ps --no-heading cft?' | $vprefix bash | grep -v '\\_' | awk '{print $NF}') <(echo "$listen_services") | 
	grep -v  '^-$' | sed 's/.pid$//' | grep -v '^[0-9]*$' | 
	while read s; do
		[ -z "$s" ] && continue
		for e in $EXCLUDE_SERVICES; do
			[ "$s" = "${e:0:15}" ] && continue 2
		done
		echo "$s"
	done

}

find_missing_services_func () {
	sleep 2
	search_running_services_func >$services_after
	diff $services_before $services_after >$services_diff
	missing_servs=`grep '^<' $services_diff | cut -d' ' -f2`
}

trap int_func SIGINT

#[ -z "$vservers" ] && vservers=`cat $VSERVER_CONFIG_DIR/*/name`
[ -z "$vservers" ] && vservers=`ls $VSERVER_CONFIG_DIR`
for v in $vservers; do 
	vserver $v running && {
		context=`cat $VSERVER_CONFIG_DIR/$v/context`
	        vprefix=""
	        [ -n "$v" ] && vprefix="vserver $v exec"
		apt_command=apt-get
		[ "`$vprefix cat /etc/debian_version | cut -d. -f1`" = 5 ] && apt_command=aptitude 
		echo
		echo $v - update
		echo '--------' 
		$CHECK_ETC && $vprefix /bin/bash -c "type check-etc-rcs-files" && $vprefix check-etc-rcs-files --before-dist-upgrade
		[ $? -eq $ABORT_CODE ] && ask_cont
		interrupted=false
		#vserver $v exec aptitude update
		$vprefix apt-get update
                status=$?
                if [ $status -ne 0 ] ; then
                        update_fehler_liste+="  $v"
			if ! $interrupted; then
                        	ask_cont "aptitude update endet mit Fehler-Status $status"
			fi
                fi
		$vprefix apt-get -V -y  install lihas-keyring
		interrupted=false
		echo $v - upgrade
		echo '========' 
		$CHECK_SERVICES && search_running_services_func >$services_before
		#export UCF_FORCE_CONFFOLD=1 # nicht immer richtig
		#  vserver $v exec aptitude -V -o Dpkg::Options::="--force-confdef" -o Dpkg::Options::="--force-confold"  "${apt_args[@]}"  dist-upgrade
		if [ ${#apt_args[*]} -eq 0 ]; then
			$vprefix  $apt_command -V dist-upgrade
		else
			$vprefix  $apt_command -V "${apt_args[@]:-}" dist-upgrade
		fi
                status=$?
                if [ $status -ne 0 ] ; then
                        upgrade_fehler_liste+="  $v"
			if ! $interrupted; then
                        	ask_cont "$apt_command dist-upgrade endet mit Fehler-Status $status"
			fi
                fi
		interrupted=false
		$CHECK_ETC && $vprefix  /bin/bash -c "type check-etc-rcs-files" && $vprefix  check-etc-rcs-files --after-dist-upgrade
		[ $? -eq $ABORT_CODE ] && ask_cont
		#vserver $v exec aptitude clean
		$vprefix  apt-get clean
		if $CHECK_SERVICES ; then 
			find_missing_services_func
			if [ -n "$missing_servs" ] ; then
				if $RESTART_SERVICES; then
					for s in $missing_servs; do
						[ "$s" = master ] && s=postfix
						if $vprefix  [ -x /etc/init.d/$s ] ; then
							$vprefix  /etc/init.d/$s start 
						elif s=${s%-*}; $vprefix  [ -x /etc/init.d/$s ] ; then
							$vprefix  /etc/init.d/$s start
						fi
						started_services+="$v:$s "
					done
					find_missing_services_func
				fi
				if [ -n "$missing_servs" ] ; then
					ask_cont "gestoppte Dienste nach dem Upgrade: "$missing_servs
				fi
			fi
			new_servs=`grep '^>' $services_diff | cut -d' ' -f2` 
			if [ -n "$new_servs" ] ; then
				ask_cont "neu gestartete Dienste nach dem Upgrade: "$new_servs
			fi
		fi
	} 
done

endausgabe_func

